@@ -1,5 +1,4 @@
 # -*- python-mode -*-
-# -*- coding: UTF-8 -*-
 
 ## Copyright (C) 2012-2013  Daniel Pavel
 ##
@@ -17,11 +16,10 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import time
 
 from collections import namedtuple
+from logging import DEBUG as _DEBUG
 from logging import INFO as _INFO
 from logging import WARNING as _WARNING
 from logging import getLogger
@@ -72,7 +70,7 @@ class ReceiverListener(_listener.EventsListener):
     """Keeps the status of a Receiver.
     """
     def __init__(self, receiver, status_changed_callback):
-        super(ReceiverListener, self).__init__(receiver, self._notifications_handler)
+        super().__init__(receiver, self._notifications_handler)
         # no reason to enable polling yet
         # self.tick_period = _POLL_TICK
         # self._last_tick = 0
@@ -193,6 +191,12 @@ class ReceiverListener(_listener.EventsListener):
             _notifications.process(self.receiver, n)
             return
 
+        # a notification that came in to the device listener - strange, but nothing needs to be done here
+        if self.receiver.isDevice:
+            if _log.isEnabledFor(_DEBUG):
+                _log.debug('Notification %s via device %s being ignored.', n, self.receiver)
+            return
+
         # a device notification
         if not (0 < n.devnumber <= self.receiver.max_devices):
             if _log.isEnabledFor(_WARNING):
@@ -220,7 +224,7 @@ class ReceiverListener(_listener.EventsListener):
             return
         elif n.sub_id == 0x41:
             if not already_known:
-                if n.address == 0x0A:
+                if n.address == 0x0A and not self.receiver.receiver_kind == 'bolt':
                     # some Nanos send a notification even if no new pairing - check that there really is a device there
                     if self.receiver.read_register(_R.receiver_info, _IR.pairing_information + n.devnumber - 1) is None:
                         return
