@@ -1,5 +1,4 @@
 # -*- python-mode -*-
-# -*- coding: UTF-8 -*-
 
 ## Copyright (C) 2012-2013  Daniel Pavel
 ##
@@ -17,8 +16,6 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import os
 
 from logging import DEBUG as _DEBUG
@@ -34,6 +31,7 @@ from solaar import NAME
 from solaar.i18n import _
 
 from . import icons as _icons
+from .about import show_window as _show_about_window
 from .window import popup as _window_popup
 from .window import toggle as _window_toggle
 
@@ -44,7 +42,7 @@ del getLogger
 # constants
 #
 
-_TRAY_ICON_SIZE = 32  # pixels
+_TRAY_ICON_SIZE = 64  # pixels - make large as downscaling is done but not always upscaling
 _MENU_ICON_SIZE = Gtk.IconSize.LARGE_TOOLBAR
 _RECEIVER_SEPARATOR = ('~', None, None, None)
 
@@ -63,10 +61,10 @@ def _create_menu(quit_handler):
     menu.append(no_receiver)
     menu.append(Gtk.SeparatorMenuItem.new())
 
-    from .action import about, make
-    menu.append(about.create_menu_item())
-    menu.append(make('application-exit', _('Quit') + ' ' + NAME, quit_handler, stock_id='application-exit').create_menu_item())
-    del about, make
+    from .action import make
+    menu.append(make('help-about', _('About %s') % NAME, _show_about_window, stock_id='help-about').create_menu_item())
+    menu.append(make('application-exit', _('Quit %s') % NAME, quit_handler, stock_id='application-exit').create_menu_item())
+    del make
 
     menu.show_all()
 
@@ -174,10 +172,11 @@ try:
 
     # Defense against AppIndicator3 bug that treats files in current directory as icon files
     # https://bugs.launchpad.net/ubuntu/+source/libappindicator/+bug/1363277
+    # Defense against bug that shows up in XFCE 4.16 where icons are not upscaled
     def _icon_file(icon_name):
-        if not os.path.isfile(icon_name):
+        if False and not os.path.isfile(icon_name):
             return icon_name
-        icon_info = Gtk.IconTheme.get_default().lookup_icon(icon_name, _TRAY_ICON_SIZE, 0)
+        icon_info = Gtk.IconTheme.get_default().lookup_icon(icon_name, _TRAY_ICON_SIZE, Gtk.IconLookupFlags.FORCE_SVG)
         return icon_info.get_filename() if icon_info else icon_name
 
     def _create(menu):
@@ -352,27 +351,27 @@ def _pick_device_with_lowest_battery():
 #
 
 
+def _add_separator(index):
+    _devices_info.insert(index + 1, _RECEIVER_SEPARATOR)
+    separator = Gtk.SeparatorMenuItem.new()
+    separator.set_visible(True)
+    _menu.insert(separator, index + 1)
+
+
 def _add_device(device):
     assert device
-    # not true for wired devices - assert device.receiver
-    receiver_path = device.receiver.path if device.receiver is not None else device.path
-    # not true for wired devices - assert receiver_path
 
     index = 0
-    for idx, (path, _ignore, _ignore, _ignore) in enumerate(_devices_info):
-        if path == receiver_path:
-            # the first entry matching the receiver serial should be for the receiver itself
-            index = idx + 1
-            break
-    # assert index is not None
-
-    if device.receiver:
-        # proper ordering (according to device.number) for a receiver's devices
-        while True:
+    receiver_path = device.receiver.path if device.receiver is not None else device.path
+    if device.receiver is not None:  # if receiver insert into devices for the receiver in device number order
+        for idx, (path, _ignore, _ignore, _ignore) in enumerate(_devices_info):
+            if path and path == receiver_path:
+                index = idx + 1  # the first entry matching the receiver serial should be for the receiver itself
+                break
+        while index < len(_devices_info):
             path, number, _ignore, _ignore = _devices_info[index]
             if path == _RECEIVER_SEPARATOR[0]:
                 break
-            assert path == receiver_path
             assert number != device.number
             if number > device.number:
                 break
@@ -382,9 +381,7 @@ def _add_device(device):
     assert len(new_device_info) == len(_RECEIVER_SEPARATOR)
     _devices_info.insert(index, new_device_info)
 
-    # label_prefix = b'\xE2\x94\x84 '.decode('utf-8')
     label_prefix = '   '
-
     new_menu_item = Gtk.ImageMenuItem.new_with_label((label_prefix if device.number else '') + device.name)
     new_menu_item.set_image(Gtk.Image())
     new_menu_item.show_all()
@@ -408,23 +405,20 @@ def _remove_device(index):
 
 
 def _add_receiver(receiver):
-    index = len(_devices_info)
+    index = 0
 
     new_receiver_info = (receiver.path, None, receiver.name, None)
     assert len(new_receiver_info) == len(_RECEIVER_SEPARATOR)
-    _devices_info.append(new_receiver_info)
+    _devices_info.insert(0, new_receiver_info)
 
     new_menu_item = Gtk.ImageMenuItem.new_with_label(receiver.name)
-    _menu.insert(new_menu_item, index)
     icon_set = _icons.device_icon_set(receiver.name)
     new_menu_item.set_image(Gtk.Image().new_from_icon_set(icon_set, _MENU_ICON_SIZE))
     new_menu_item.show_all()
     new_menu_item.connect('activate', _window_popup, receiver.path)
+    _menu.insert(new_menu_item, index)
 
-    _devices_info.append(_RECEIVER_SEPARATOR)
-    separator = Gtk.SeparatorMenuItem.new()
-    separator.set_visible(True)
-    _menu.insert(separator, index + 1)
+    _add_separator(index)
 
     return 0
 
@@ -448,7 +442,7 @@ def _remove_receiver(receiver):
 
 
 def _update_menu_item(index, device):
-    if not device or device.status is None:
+    if device is None or device.status is None:
         _log.warn('updating an inactive device %s, assuming disconnected', device)
         return None
 
